#include <QMessageBox>
#include <QDebug>
#include <QUrl>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QTime>
#include <QTimer>

#include "ui_mainwindow.h"

#include "mainwindow.h"
#include "version.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    connect(ui->action_FS, SIGNAL(triggered()), this, SLOT(showFullScreen()));
    connect(ui->action_Max, SIGNAL(triggered()), this, SLOT(showMaximized()));
    connect(ui->action_Normal, SIGNAL(triggered()), this, SLOT(showNormal()));
    connect(ui->actionClose, SIGNAL(triggered()), this, SLOT(close()));
    connect(ui->action_QT, SIGNAL(triggered()), qApp, SLOT(aboutQt()));


QStringList sl;
sl = QCoreApplication::arguments ();
qDebug() << sl;

if (sl.contains("-fs", Qt::CaseInsensitive))
    showFullScreen();

if (sl.contains("-max", Qt::CaseInsensitive))
    showMaximized();

QTimer *timer = new QTimer(this);
connect(timer, SIGNAL(timeout()), this, SLOT(update_status()));
timer->start(1000);

if (sl.size() < 2)
{
    qDebug("Nothing to display");
    return;
}

int repeat = sl.indexOf("-r");
if (repeat>0 && (repeat < sl.size()))
{
    bool ok;
    int seconds = 1000 * sl[repeat +1].toInt(&ok, 10);
    if (ok)
    {
        QTimer *t = new QTimer(this);
        connect(t, SIGNAL(timeout()), this, SLOT(load_image()));
        t->start(seconds);
        qDebug() << "Periodic loading each" << seconds << "seconds";
    }
    else
    {
        qWarning("Parameter following -r must be a number of seconds");
    }
}

file_ = sl[sl.size() - 1];
load_image();
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::changeEvent(QEvent *e)
{
    QMainWindow::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainWindow::load_image()
{
    ui->statusBar->showMessage("Load file : " + file_);
    if (file_.startsWith("http://"))
    {
        get_image(file_);
    }
    else
    {
        QPixmap pix(file_);
        display_image(pix);
    }
}

void MainWindow::on_action_A_propos_triggered()
{
    QMessageBox::about(this,
                         tr("About"),
                         tr("<p><b>qdisplay</b></p>"
                            "<p>Version qdisplay: %1"
                            "<p>Version %2: %3.%4"
                            "</p>")
                         .arg(VERSION)
  #ifdef __GNUC__
                         .arg("GCC")
                         .arg(__GNUC__)
                         .arg(__GNUC_MINOR__)
  #elif defined(_MSC_VER)
                         .arg("VC++")
                         .arg(_MSC_VER / 100)
                         .arg(_MSC_VER % 100,2,10,QChar('0'))
  #else
                         .arg("Compiler")
                         .arg("?")
                         .arg("?")
  #endif
                         );
}

void MainWindow::display_image(const QPixmap & pix)
{
    if (pix.isNull())
    {
        ui->toto->setPixmap(QPixmap(QString::fromUtf8(":/file_broken.png")));
    }
    else
    {
        ui->toto->setPixmap(pix);
    }
}

void MainWindow::get_image(const QString & s)
{
    QNetworkAccessManager *manager = new QNetworkAccessManager(this);
     connect(manager, SIGNAL(finished(QNetworkReply*)), this, SLOT(replyFinished(QNetworkReply*)));
     manager->get(QNetworkRequest(QUrl(s,QUrl::TolerantMode)));
     qDebug()<< "url submited :        " << s;
}

void MainWindow::replyFinished(QNetworkReply * reply)
{
    QByteArray ba;
    ba = reply->readAll();
    reply->manager()->deleteLater();
    reply->deleteLater();

    qDebug()<< "url corrected: " << reply->url();

    QPixmap p;
    p.loadFromData(ba);
    display_image(p);
}

void MainWindow::update_status()
{
    QTime t;
    t = QTime::currentTime();
    ui->statusBar->showMessage(t.toString("HH:mm:ss"));
}
